(** Time-stamp: <modified the 14/03/2008 (at 11:11) by Erwan Jahier> *)

(**********************************************************
Sous-module pour SyntaxTab 
************************************************************

   Une SymbolTab.t est une association ``contextuelle'' entre
des ident simple (string) et ce  quoi ils correspondent.
   N.B. il est inutile de grer les idents absolus (pack+nom)
qui sont par dfinition NON-CONTEXTUELS.
   Dans une table, on a 3 espaces de noms, un par nature d'item
(const, type et oper).

***********************************************************)
open SyntaxTreeCore

type 'a hereflagged =
    Here of 'a            (* local items *) 
  | NotHere of Ident.long (* imported items *)

type t

val create : unit -> t

(* Manip de SymbolTab.t *)

(* Raise a proper compil error message if not found *)
val find_type :  t -> Ident.t -> Lxm.t -> (type_info  Lxm.srcflagged) hereflagged
val find_const : t -> Ident.t -> Lxm.t -> (const_info Lxm.srcflagged) hereflagged
val find_node :  t -> Ident.t -> Lxm.t -> (node_info  Lxm.srcflagged) hereflagged

(* Ajout de nom d'item imports (via uses) *)
val add_import_const : t -> Ident.t -> Ident.long -> unit
val add_import_type  : t -> Ident.t -> Ident.long -> unit
val add_import_node  : t -> Ident.t -> Ident.long -> unit

(** Add local items declaration *)
val add_type  : t -> Ident.t -> type_info  Lxm.srcflagged -> unit
val add_const : t -> Ident.t -> const_info Lxm.srcflagged -> unit
val add_node  : t -> Ident.t -> node_info  Lxm.srcflagged -> unit

(* Itrer sur les items *)

val iter_types : 
  t -> (Ident.t -> (type_info Lxm.srcflagged) hereflagged -> unit) -> unit
val iter_consts : 
  t -> (Ident.t -> (const_info Lxm.srcflagged) hereflagged -> unit) -> unit
val iter_nodes : 
  t -> (Ident.t -> (node_info Lxm.srcflagged) hereflagged -> unit) -> unit

val dump : t -> unit

